function [EEG, command] = pop_importNDF(foldername, varargin)
  %
  % pop_importNDF() - import data files into EEGLAB using neuracle toolbox
  %
  % Usage:
  %   >> [EEG, command] = pop_importNDF(); % pop up window
  %   >> [EEG, command] = pop_importNDF( foldername, 'key', val, ...);
  %   >> [EEG, command] = pop_importNDF( foldername, 'key', val, ...);
  %   >> [EEG, command] = pop_importNDF('foldername','DataType','ECG','channels',{'probe2_ECG'},'blockrange',[0 100]);
  %   >> [EEG, command] = pop_importNDF('foldername','DataType','EEG','channels',{'Fp1'},'blockrange',[0 100]);
  %   >> [EEG, command] = pop_importNDF('foldername','DataType','EEG','channels',[1:10],'blockrange',[0 100]);

  %
  % Inputs:
  %   foldername - [string] folder name,if folder is the experiment fold
  %   which includes many subfolder,it will pop a subject selecting
  %   window,else if the folder is the subject folder,it will not pop the
  %   subject selecting window
  %
  % Optional inputs:
  %   1.'DataType'   -['EEG(EOG/HECG)'|'EMG/HEMG'|'ECG'|'9-Axis'|'GSR'|'Pulse-SPO2-HR'|'ALL']
  %   default pop select window.If ALL is selected, it will return all signals
  %   upsampled to the highest frequency.9-Axis includes
  %   GYR-X,Y,Z,ACC-X,Y,Z,MAG-X,Y,Z channel.
  %
  %   2.'channels'   - [integer array] list of channel indices
  %   'blockrange' - [min max] integer range of data blocks to import, in seconds.
  %                  Entering [0 3] will import the first three blocks of data.
  %                  Default is empty -> import all data blocks.
  %  3.'importevent' - ['on'|'off'] import events. Default if 'on'.
  %  4.'memorymapped' - ['on'|'off'] import memory mapped file (useful if
  %                  encountering memory errors). Default is 'off'.
  % Outputs:
  %   EEG   - EEGLAB data structure
  %   command - the program command of read data

  % dubug at 20231220  by shencai hao,删除字段元素为空的元素 解决上位机删除数据中的trigger引起的读取事件失败问题
  % fixed at 20240105 by shencai hao,  HEMG单独作为一个类型，跟EMG分开


  EEG = [];
  command = '';
  g = struct('DataType',[]);
  if nargin < 1
    folder = uigetdir('*.*', 'Choose a folder -- pop_importNDF()');
    if isequal(folder,0)
      disp('Cancel choosing a folder...');
      return;
    end
    foldername = folder;

  else
    % 直接给定路径信息
    folder = foldername;
    options = varargin;
    g = finputcheck( options, { 'blockrange'   'integer' [0 Inf]    [];
                                'channels'   { 'integer','cell' }  [0 Inf]  [];
                                'DataType'      'string' {}         [];
                                'importevent'  'string'  { 'on';'off' } 'on';
                                'memorymapped' 'string'  { 'on';'off' } 'off';});
    if ischar(g), error(g); end

  end

  nefPath = '';
  nefDir = dir(fullfile(foldername, '*.nef'));
  if ~isempty(nefDir)
    nefPath = fullfile(folder, nefDir(1).name);
  end

  nfiDir = dir(fullfile(foldername, '*.nfi'));
  if length(nfiDir) < 1
    disp('No *.nfi file was found in chosen folder.');
    return;
  end

  nfiName = nfiDir(1).name;
  folderCat = fileNFI.GetFolderCategory(fullfile(folder, nfiName));

  switch folderCat
    case FolderType.NDFFileTypeNTP
      % 选择实验文件夹
      ntps = dir(fullfile(foldername, '*.ntp'));
      if length(ntps) < 1
        disp('No *.ntp file was found in chosen folder.');
        return;
      end
      ntpName = fullfile(folder,ntps(1).name);
      ntp = fileNTP(ntpName);
      ntpInfo = ntp.parse();
      %fileNTPInfo.recordDate
      subs = ntpInfo.subs;
      cnt = ntpInfo.subCount;
      subName = strings(1,cnt);
      choiceStr = '';
      if cnt > 1
        for i = 1:cnt-1
          sname = subs(i).folderName;
          choiceStr = strcat(choiceStr, sname,"|");
        end
      end
      choiceStr = strcat(choiceStr, subs(cnt).folderName);
      res = inputgui( 'geometry', { [1 1] }, ...
                      'geomvert', cnt, 'uilist', { ...
                       { 'style', 'text', 'string', [ 'Make a choice' 10 10 10 ] }, ...
                       { 'style', 'listbox', 'string', choiceStr } } );
      subI = res{1};
      if(subI<1)
        disp("Cancel...");
        return;
      else
        sname = subs(subI).folderName;
        foldername = fullfile(folder,sname);
        fldObj = FolderNHF(foldername);
        % 20230718 增加单类型参数选择功能
        if isempty(g.DataType)
          [SelChType,SelCh,SelChSrate] = extract_one_type(fldObj);
        else
          [SelChType,SelCh,SelChSrate] = extract_one_type(fldObj,g.DataType);
        end
        if isempty(SelChType)
          return;
        end
        fldInfo = fldObj.getFolderInfo(SelChType,SelCh,SelChSrate);%20230718
      end

    case FolderType.NDFFileTypeNHF
      % 选择被试文件夹
      fldObj = FolderNHF(foldername);
       % 20230718 增加单类型参数选择功能
      if isempty(g.DataType)
        [SelChType,SelCh,SelChSrate] = extract_one_type(fldObj);
      else
        [SelChType,SelCh,SelChSrate] = extract_one_type(fldObj,g.DataType);
      end
      if isempty(SelChType)
        return;
      end
      fldInfo = fldObj.getFolderInfo(SelChType,SelCh,SelChSrate);%20230718
        
    case FolderType.NDFFileTypeNSF
      % 选择被试文件夹下一层数据文件夹
      fldObj = FolderNSF(foldername);
      fldInfo = fldObj.getFolderInfo();
  end

  %%
  if length(varargin) < 2
    eventOnOff = 1;
    eeglab_options;
    uilist = { { 'style' 'text' 'String' ['Channel list (defaut all [1 ' num2str(fldInfo.channelCount) '])'] } ...
                 { 'style' 'edit' 'string' '' } ...
                 { 'style' 'text' 'String' [ 'Data range (in seconds) to read (default all [0 ' num2str(fldInfo.duration) '])' ] } ...
                 { 'style' 'edit' 'string' '' } ...
                 { 'style' 'text' 'String' 'Extract event' } ...
                 { 'style' 'checkbox' 'string' '' 'value' eventOnOff 'enable' 'on' } {} ...
                 { 'style' 'checkbox' 'String' 'Import as memory mapped file (use if out of memory error)' 'value' option_memmapdata } };
    geom = { [3 1] [3 1] [3 0.35 0.5] [1] };
    result = inputgui( geom, uilist, 'pophelp(''pop_importNDF'')', 'Load data using Neuracle-IO -- pop_importNDF()');
    if isempty(result)
      return;
    end
    % decode GUI params
    % -----------------

    options = {};
    try
      eval([result{1}]);
    catch
      chanlabels = lower(fldInfo.channelNames);
      stridx = lower(strsplit(result{1},' '));
      [Lia,Locb] = ismember(stridx,chanlabels);
      result{1} = int2str(Locb(Lia)');
      if ~isempty(find(Lia == 0))
        disp(['can not find channel: ' stridx{~Lia}]);
      end
    end

    if ~isempty(result{1})
      options = { options{:} 'channels'   eval([ '[' result{1} ']' ]) }
    end

    if ~isempty(result{2})
      options = { options{:} 'blockrange' eval( [ '[' result{2} ']' ] ) };
    end

    if length(result) > 2
      if ~result{3}
          options = { options{:} 'importevent'  'off'  };
      end
      if  result{4}
          options = { options{:} 'memorymapped' 'on' };
      end
    end

    % decode imput parameters
    % -----------------------
    optStr = struct(options{:});
    if ~isfield(optStr, 'blockrange')
      optStr.blockrange = [];
    end
    if ~isfield(optStr, 'channels')
      optStr.channels = [];
    end
    if ~isfield(optStr, 'importevent')
      if eventOnOff == 0
        optStr.importevent = 'off';
      else
        optStr.importevent = 'on';
      end
    end
    if ~isfield(optStr, 'memorymapped')
      optStr.memorymapped = 'off';
    end

  else
    optStr.blockrange = g.blockrange;
    optStr.channels = g.channels;
    optStr.importevent = g.importevent;
    optStr.memorymapped = g.memorymapped;
  end


  % import event
  if strcmpi(optStr.importevent,'on')
    event = struct('type',[],'latency',[]);
    eIdex = 0;
    if ~isempty(nefPath)
      nefObj = fileNEF(nefPath);
      evtInfo = nefObj.nefInfo;
      evtCnt = evtInfo.eventCount -  evtInfo.eventDelCount;
      evtAr = evtInfo.eventAr;
      % debug 上位机删除事件导致事件文件对应为空，数量对不上问题
        indicesToDelete = [];
        for i = 1:numel(evtAr)
            if isempty(evtAr(i).Annotation)
                indicesToDelete = [indicesToDelete, i];
            end
        end
        evtAr(indicesToDelete) = [];

      for i = 1:evtCnt
        event(i).type = evtAr(i).Annotation;
        event(i).latency = evtAr(i).Onset*0.001*fldInfo.sRate;% round(evt(i).offset_in_sec*srate);
        %event(i).latency = evtAr(i).Onset*fldInfo.sRate;% round(evt(i).offset_in_sec*srate);
      end
      eIdex = eIdex+evtCnt;
    end

    segs = fldInfo.recordSegs;
    segCnt = fldInfo.segCount;
    if segCnt>1
      for i = 1:segCnt
        seg = segs(i);
        idx = eIdex + i;
        typeS = sprintf("%s record segment %d",seg.SegmentFolderName,seg.Index);
        event(idx).type = char(typeS);
        %event(idx).latency = seg.StartSecond*0.001*fldInfo.sRate;% round(evt(i).offset_in_sec*srate);
        event(idx).latency = seg.StartSecond*fldInfo.sRate;% round(evt(i).offset_in_sec*srate);
      end
    end
    Event = event;
  else
    Event = struct('type',[],'latency',[]);
  end

  %%
  % import data
  % -----------
  EEG = eeg_emptyset;
  EEG.srate = fldInfo.sRate;
  % it is continuous data, therefore append all records in one trial
  EEG.trials = 1;
  EEG.ref = 'common';
  chaninfo.plotrad = [];
  chaninfo.shrink = [];
  chaninfo.nosedir = '+X';
  chaninfo.icachansind = [];
  EEG.chaninfo = chaninfo;
  patientName = strtrim(fldInfo.patientName);
  if ~strcmpi(patientName,'X')
      EEG.setname = [EEG.setname ' ' patientName ];
  end
  % organize EEG info
  % -------------
  if ~isempty(optStr.channels)
    if isa(optStr.channels,'double')
      chNames = fldInfo.channelNames(optStr.channels);
    else
      chNames = optStr.channels;
    end
  else
    chNames = fldInfo.channelNames;
    optStr.channels = chNames;
  end

  chanlocsAr = struct([]);
  chL = length(chNames);
  for i = 1:chL
    chanlocs = struct('labels',char(chNames(i)),'ref',[],'theta',[],'radius',[],'X',[],'Y',[],'Z',[],'sph_theta',[],'sph_phi',[],'sph_radius',[],'type',[],'urchan',[] );
    chanlocsAr = [chanlocsAr;chanlocs];
  end
  EEG.chanlocs =  chanlocsAr;
  EEG.nbchan = length(chNames);
  EEG.pnts = fldInfo.duration*fldInfo.sRate;
  EEG.xmax = EEG.pnts/fldInfo.sRate;
  EEG.times = [1:EEG.pnts]*1000/fldInfo.sRate;
  % read data
  % ----------
  fprintf('Reading data ...\n');
  %%%%%
  importOp = EEGDataOption(foldername);

  if isempty(optStr.channels)||(fldInfo.channelCount==length(optStr.channels))
    importOp.isAllChannels=true;
    importOp.channelNames = optStr.channels;

  else
    importOp.isAllChannels=false;
    importOp.channelNames = chNames;
  end

  begsample = 1;
  endsample = EEG.pnts;
  if isempty(optStr.blockrange)
    importOp.isAllTimeRange=true;
    importOp.startSec=0;
    importOp.duration=fldInfo.duration;
  else
    importOp.isAllTimeRange=false;
    importOp.startSec=optStr.blockrange(1);
    begsample = optStr.blockrange(2);
    importOp.duration=optStr.blockrange(end) - optStr.blockrange(1);
  end

  [~,I] = sort([Event(:).latency]);
  Event = Event(I);
  latency = [Event(:).latency];
    
    %% dubug 20231220 删除字段元素为空的元素
    indicesToDelete = [];
    for i = 1:numel(Event)
        if isempty(Event(i).type) || isempty(Event(i).latency)
            indicesToDelete = [indicesToDelete, i];
        end
    end
    Event(indicesToDelete) = [];

  [Lia,~] = ismember( {Event(:).type},'boundary');
  boundarylatency = latency(Lia);
  idxEvent =  find(begsample<= latency & latency <= endsample);
  if ~isempty(idxEvent)
    for k = 1:length(idxEvent)
      EEG.event(k).type = Event(idxEvent(k)).type;
      EEG.event(k).latency = Event(idxEvent(k)).latency - begsample +1;
    end
  end

  EEG.data = fldObj.getFolderData(importOp);
  % 返回操作指令
  if length(foldername) == 1
    command = sprintf('EEG = pop_importNDF(''%s''', foldername{1});
  else
    command = sprintf('EEG = pop_importNDF(%s', vararg2str(foldername));
  end

  if isempty(options)
    command = [ command ');' ];
  else
    try
        command = [ command sprintf(', %s);',vararg2str(options))];
    catch
        SelCh = [];
        for i=1:length(options)
            if strncmpi('channels',options{i},5)
                SelCh = i+1;
            end
        end
        if isa(options{SelCh},'string')
        
            chname = char(options{SelCh});
        else
            chname = options{SelCh};
            for j = 1:length(chname)
                chname{j} = char(chname{j});
            end
        end
        options{SelCh} = eval(['{', vararg2str(chname) ,'}']);
        command = [ command sprintf(', %s);',vararg2str(options))];
    end  
  end
  EEG = eeg_checkset(EEG);
  EEG = eeg_hist(EEG, command );

end

function [SelChType,SelCh,SelChSrate] = extract_one_type(fldObj,varargin)
  nVarargs = length(varargin);%用于判断是否是脚本程序

  % 查询数据中有哪些参数
  Chtype = {fldObj.fldNHFHeader.chs.ChannelType};
  for i=1:length(Chtype)
    Chtype{i} = Chtype{i}';
  end
  %相同参数类型合并
  [chName,ia,~] = unique(Chtype);
  ChType_set = {'EEG','EMG','HEMG','ECG','ACC','GSR','Pulse','ALL'};%目前数据类型有八种
  for i=1:length(ChType_set)
    try
      iloc = ia(ChType_exist(ChType_set{i},chName));%定位参数位置
      srate_Type(i) = fldObj.fldNHFHeader.sampleRateAr(iloc);%获取对应采样率
    catch
      srate_Type(i) = 0;%如果该参数不存在，则采样率设为零
    end
  end

  if nVarargs==0
    choiceStr = 'EEG(EOG/HECG)|EMG|HEMG|ECG|9-Axis|GSR|Pulse-SPO2-HR|ALL';
    Information = ['Here is the descripiton of the Data:',...
                    newline,'1.EEG(EOG/HECG):          ', num2str(srate_Type(1)),'Hz' ,...
                    newline,'2.EMG:          ',num2str(srate_Type(2)),' Hz' ,...
                    newline,'3.HEMG:          ',num2str(srate_Type(3)),' Hz' ,...
                    newline,'4.ECG:          ' ,num2str(srate_Type(4)),'Hz',...
                    newline,'5.9-Axis:       ' ,num2str(srate_Type(5)),'Hz',...
                    newline,'6.GSR:          ' ,num2str(srate_Type(6)),'Hz',...
                    newline,'7.Pulse-SPO2-HR:' ,num2str(srate_Type(7)),'Hz',...
                    newline,'If ALL is selected, it will return all signals upsampled to the highest frequency!',...
                    newline,'Notice: 0 Hz represents the absence of this type of signal!'];
    res = inputgui( 'geometry', { [1 1] }, ...
                    'geomvert', 9, 'uilist', { ...
                      { 'style', 'text', 'string', Information }, ...
                      { 'style', 'listbox', 'string', choiceStr } });
    subI = res{1};% 选择类型
  else
    if strcmp(varargin{1},'9-Axis')
        varargin{1} = 'ACC';
    end
    % 脚本输入
    subI = findcell(ChType_set,varargin{1});
  end

  SelChType = [];%选择参数类型
  SelCh = [];%选择参数类型对应通道位置
  SelChSrate = [];%参数对应采样率

  switch subI
    case 1
      if ChType_exist('EEG',{fldObj.fldNHFHeader.chs.ChannelType})||ChType_exist('EOG',{fldObj.fldNHFHeader.chs.ChannelType})||ChType_exist('HECG',{fldObj.fldNHFHeader.chs.ChannelType})%1判断数据中有没有脑电
        %提取脑电数据
        for i=1:fldObj.fldNHFHeader.chCountsNHFLevel
            SelCh = [SelCh strncmpi('EEG',{fldObj.fldNHFHeader.chs(i).ChannelType},3)*i];
            SelCh = [SelCh strncmpi('EOG',{fldObj.fldNHFHeader.chs(i).ChannelType},3)*i];
            SelCh = [SelCh strncmpi('HECG',{fldObj.fldNHFHeader.chs(i).ChannelType},3)*i];

        end
        SelCh = SelCh(SelCh>0);
        SelChType = 'EEG';
        SelChSrate = srate_Type(1);
      else
        disp('Can not find EEG type signal: ');%没有该类型数据报错
        return;
      end

      case 2
        if ChType_exist('EMG',{fldObj.fldNHFHeader.chs.ChannelType})  %1判断数据中有没有肌电
         %提取肌电数据
          for i=1:fldObj.fldNHFHeader.chCountsNHFLevel
            SelCh = [SelCh strncmpi('EMG',{fldObj.fldNHFHeader.chs(i).ChannelType},3)*i];
          end
          SelCh = SelCh(SelCh>0);
          SelChType = 'EMG';
          SelChSrate = srate_Type(2);
        else
          disp('Can not find EMG type signal: ');%没有该类型数据报错
          return;
        end


      case 3
        if ChType_exist('HEMG',{fldObj.fldNHFHeader.chs.ChannelType})  %1判断数据中有没有肌电
         %提取肌电数据
          for i=1:fldObj.fldNHFHeader.chCountsNHFLevel
            SelCh = [SelCh strncmpi('HEMG',{fldObj.fldNHFHeader.chs(i).ChannelType},3)*i];
          end
          SelCh = SelCh(SelCh>0);
          SelChType = 'HEMG';
          SelChSrate = srate_Type(3);
        else
          disp('Can not find HEMG type signal: ');%没有该类型数据报错
          return;
        end


      case 4
         if ChType_exist('ECG',{fldObj.fldNHFHeader.chs.ChannelType})  %1判断数据中有没有心电
         %提取心电数据
          for i=1:fldObj.fldNHFHeader.chCountsNHFLevel
            SelCh = [SelCh strncmpi('ECG',{fldObj.fldNHFHeader.chs(i).ChannelType},3)*i];
          end
          SelCh = SelCh(SelCh>0);
          SelChType = 'ECG';
          SelChSrate = srate_Type(4);
        else
          disp('Can not find ECG type signal: ');%没有该类型数据报错
          return;
        end

      case 5
        if ChType_exist('ACC',{fldObj.fldNHFHeader.chs.ChannelType})  %1判断数据中有没有九轴数据，
         %提取九轴数据
          for i=1:fldObj.fldNHFHeader.chCountsNHFLevel
            SelCh = [SelCh strncmpi('GYR',{fldObj.fldNHFHeader.chs(i).ChannelType},3)*i];
            SelCh = [SelCh strncmpi('GRY',{fldObj.fldNHFHeader.chs(i).ChannelType},3)*i];% 不同数据中出现名称不一致时
            SelCh = [SelCh strncmpi('ACC',{fldObj.fldNHFHeader.chs(i).ChannelType},3)*i];
            SelCh = [SelCh strncmpi('MAG',{fldObj.fldNHFHeader.chs(i).ChannelType},3)*i];
          end
          SelCh = SelCh(SelCh>0);
          SelChType = '9-Axis';
          SelChSrate = srate_Type(5);
        else
          disp('Can not find 9-Axis type signal: ');%没有该类型数据报错
          return;
        end

      case 6
        if ChType_exist('GSR',{fldObj.fldNHFHeader.chs.ChannelType})  %1判断数据中有没有皮电，
        %提取皮电数据
          for i=1:fldObj.fldNHFHeader.chCountsNHFLevel
            SelCh = [SelCh strncmpi('GSR',{fldObj.fldNHFHeader.chs(i).ChannelType},3)*i];
          end
          SelCh = SelCh(SelCh>0);
          SelChType = 'GSR';
          SelChSrate = srate_Type(6);
        else
          disp('Can not find GSR type signal: ');%没有该类型数据报错
          return;
        end

      case 7 %1判断数据中有没有脉搏等数据，%提取脉搏血氧心率电数据
        if ChType_exist('Pulse',{fldObj.fldNHFHeader.chs.ChannelType})  %1判断数据中有没有九轴数据，
         %提取九轴数据
          for i=1:fldObj.fldNHFHeader.chCountsNHFLevel
            SelCh = [SelCh strncmpi('Pulse',{fldObj.fldNHFHeader.chs(i).ChannelType},3)*i];
            SelCh = [SelCh strncmpi('SPO2',{fldObj.fldNHFHeader.chs(i).ChannelType},3)*i];
            SelCh = [SelCh strncmpi('HR',{fldObj.fldNHFHeader.chs(i).ChannelType},2)*i];
          end
          SelCh = SelCh(SelCh>0);
          SelChType = 'Pulse-SPO2-HR';
          SelChSrate = srate_Type(7);
        else
          disp('Can not find Pulse-SPO2-HR type signal: ');%没有该类型数据报错
          return;
        end

      case 8 %提取全部数据
        SelCh = 1:fldObj.fldNHFHeader.chCountsNHFLevel ;
        SelChType = 'All';
        SelChSrate = max(srate_Type);
  end
end

function result = ChType_exist(type,allType)
  for i=1:length(allType)
    allType_reshape = allType{i};
    n_len = length(type);
%      allType_reshape =  reshape(allType{i},1,[]);% 20240103 debug contain函数只对字符向量行作用，修改该问题
    if strncmpi(type,allType_reshape,n_len) %|| contains(allType_reshape,type)
      result = i;
      return ;
    else
      result = 0;
    end
  end
end
